class GroundPicture < Peta::Element
  load_manifest
  belongs_to :panel
  belongs_to :picture
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :repeat, :numericality => true, :inclusion => { :in => 0..3 }
  validates :x, :numericality => true
  validates :y, :numericality => true
  validates :picture_id, :numericality => true, :existence => {:both => false}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  @@repeat_texts = ['repeat', 'repeat-x', 'repeat-y', 'no-repeat']
  
  def self.pickup_item_name
    Picture.item_name
  end
  
  def self.pickup_column_name
    self.pickup_item_name + '_id'
  end
  
  def pickup_id
    # get :picture_id if head revision
    self.attributes[self.pickup_column_name]
  end
  
  def y
    self.attributes['y']
  end
  
  def self.by_author_list_includes
    {
      :panel => {
        :author => {}
      }
    }
  end
  
  def self.has_picture?
    true
  end
  
  def supply_default
    self.x = 0
    self.y = 0
    self.repeat = 0
    if self.panel
      self.t = self.panel.new_t 
      self.z = self.panel.new_z 
    end
  end
  
  def overwrite pid
    self.panel_id = pid
  end
  
  def style spot = nil, opacity = 20
    r = {
      'position' => 'absolute', 'top' => '0px', 'left' => '0px', 'z-index' => self.z, 
      'background-image' => "url(#{self.picture.url})", 
      'background-repeat' => self.repeat_text, 
      'background-position' => "#{self.x}px #{self.y}px"
    }
    self.merge_opacity(r, opacity) if spot and spot != self
    r
  end
  
  def tmb_opt_img_tag
    tw, th = PettanImager.thumbnail_size(self.picture.width, self.picture.height)
    {:src => self.picture.url, :width => tw, :height => th, :alt => self.caption}
  end
  
  def symbol_option
    self.tmb_opt_img_tag
  end
  
  def repeat_text
    @@repeat_texts[self.repeat]
  end
  
  def self.public_list_where list
    'panels.publish > 0'
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}}, :picture => {:artist => {}, :license => {}}}}
  end
  
  def scenario
    if caption.blank?
      ''
    else
      '<p>' + ERB::Util.html_escape(self.caption) + '</p>'
    end
  end
  
  def plain_scenario
    if caption.blank?
      ''
    else
      self.caption + "\n"
    end
  end
  
end
