module Manifest
  module ControllerModule
    module ActionModule
      class Base < ManifestBase::TypeNameArgs
        attr :alias, :param_id, :item_name, :action_name, :original
        
        def set_default
          super
          @args['item_name'] ||= ::Manifest.manifest.singularize(@parent.name)
          @args['param_id'] ||= 'prefix'
        end
        
        def init
          super
          @action_name = @name
          @alias = @args['alias']
          @item_name = @args['item_name']
          @param_id = @args['param_id']
          # source manifest of alias manifest
          @original = @args['original']
        end
        
        def a_arg_names
          []
        end
        
        def b_arg_names
          ['item_name', 'param_id']
        end
        
        def controller
          @parent
        end
        
        def alias_action
          s = @alias.split('.')
          c = ::Manifest.manifest.controllers[s[0]]
          c.actions[s[1]]
        end
        
        def path_name
          ::Manifest.manifest.pluralize(@item_name)
        end
        
        def encode_query params
          q = []
          q.push('page=' + params['page'].to_s) if params['page']
          q.push('page_size=' + params['page_size'].to_s) if params['page_size']
          q.push('offset=' + params['offset'].to_s) if params['offset']
          q.push('count=' + params['count'].to_s) if params['count']
          if q.length > 0
            '?' + q.join('&')
          else
            ''
          end
        end
        
        def encode_url params = {}
          controller_name = '/' + self.path_name
          c = controller_name
          a = if @name == 'index' or @name == 'show' or @name == 'destroy'
            ''
          else
            '/' + @name
          end
          pf = params['format'] || params[:format]
          # f = if params['format']   # json request? so bad. html request as get rest api
          f = if pf == 'prof'
            '.' + params['format'].to_s
          else
            ''
          end
          pid = params['id'] || params[:id]
          i = if pid
            '/' + params['id'].to_s
          else
            ''
          end
          q = self.encode_query(params)
          case @param_id
          when 'prefix'
            c + i + a + f + q
          when 'suffix'
            c + a + i + f + q
          else
            c + a + f + q
          end
        end
        
        def url params
          self.encode_url(params)
        end
        
      end
      
    end
  end
end
