# -*- encoding: utf-8 -*-
#原画ライセンスグループ
require 'spec_helper'

describe OriginalPictureLicenseGroup do
  include ManifestsMacros
  before do
    @model = OriginalPictureLicenseGroup
  end
  
  describe 'ペタナイズに於いて' do
    it 'アイテムである' do
      expect(@model.ancestors.include?(Peta::Item)).to eq true
    end
    it '親モデルは持たない' do
      expect(@model.my_peta.parent_model_name).to be_blank
    end
    describe 'ブーストに於いて' do
      it '特筆すべき注意事項はない' do
        expect(@model.my_peta.boost.keys).to be_empty
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'original_picture_license_group'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'original_picture_license_groups'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = ['original_picture', 'license_group']
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
        describe 'original_pictureに於いて' do
          before do
            @mani = @belongs_to['original_picture']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'original_picture'
            expect(@mani.id_column).to eq 'original_picture_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq OriginalPicture
          end
        end
        describe 'license_groupに於いて' do
          before do
            @mani = @belongs_to['license_group']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'license_group'
            expect(@mani.id_column).to eq 'license_group_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq LicenseGroup
          end
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = []
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'original_picture_id', 'license_group_id', 
          'created_at', 'updated_at']
      end
      describe 'original_picture_idに於いて' do
        before do
          @attribute = @attributes['original_picture_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'license_group_idに於いて' do
        before do
          @attribute = @attributes['license_group_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
    end
    
  end
  
end
